package xyz.wbsite.dbtool.javafx.manger.callable;

import xyz.wbsite.dbtool.javafx.manger.FreeMarkerManager;
import xyz.wbsite.dbtool.javafx.manger.ManagerFactory;
import xyz.wbsite.dbtool.javafx.manger.ProjectManager;
import xyz.wbsite.dbtool.javafx.po.AbstractDBmapper;
import xyz.wbsite.dbtool.javafx.po.Api;
import xyz.wbsite.dbtool.javafx.tool.EntityReader;
import xyz.wbsite.dbtool.javafx.tool.JavaClassReader;
import xyz.wbsite.dbtool.javafx.tool.Tool;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.Callable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ApiCallable implements Callable {

    private File module;
    private File apiFile;
    private List<Api> apiList;
    private Tool tool = new Tool();

    private FreeMarkerManager freeMarkerManager;

    public ApiCallable(File module, File apiFile, List<Api> apiList) {
        this.module = module;
        this.apiFile = apiFile;
        this.apiList = apiList;
        this.freeMarkerManager = ManagerFactory.getFreeMarkerManager();
    }

    public Boolean call() throws Exception {
        if (!apiFile.exists()) {
            apiFile.mkdir();
        } else {
            Tool.clear(apiFile);
        }

        AbstractDBmapper dBmapper = ProjectManager.dBmapper;

        File domain;
        File testdomain;

        StringBuffer sbmain = new StringBuffer("");
        sbmain.append(this.apiFile.getPath() + File.separator);
        sbmain.append("src" + File.separator);
        sbmain.append("main" + File.separator);
        sbmain.append("java" + File.separator);

        StringBuffer sbtest = new StringBuffer("");
        sbtest.append(this.apiFile.getPath() + File.separator);
        sbtest.append("src" + File.separator);
        sbtest.append("test" + File.separator);
        sbtest.append("java" + File.separator);

        JavaClassReader javaClass = new JavaClassReader(apiList.get(0).getRequestFile());
        String[] split = javaClass.getDomainName().split("\\.");

        for (String s1 : split) {
            sbmain.append(s1 + File.separator);
            sbtest.append(s1 + File.separator);
        }

        domain = new File(sbmain.toString());
        domain.mkdirs();
        testdomain = new File(sbtest.toString());
        testdomain.mkdirs();

        for (Api api : apiList) {
            if (api.isCheck()) {
                File apiModule = new File(domain, api.getModule());
                apiModule.mkdirs();
                File apiRequest = new File(apiModule, "req");
                apiRequest.mkdirs();
                File apiResponse = new File(apiModule, "rsp");
                apiResponse.mkdirs();
                File entity = new File(apiModule, "ent");
                entity.mkdirs();

                List<File> entList = new ArrayList<>();
                try {
                    //region 生成request
                    JavaClassReader javaClassReader = new JavaClassReader(api.getRequestFile());
                    for (int i = 0; i < javaClassReader.getImportList().size(); i++) {
                        String s = javaClassReader.getImportList().get(i);
                        if (s.contains(".module.entity.")) {
                            javaClassReader.getImportList().set(i, s);
                        }
                    }

                    {
                        HashMap<String, Object> ctx = new HashMap<String, Object>();
                        ctx.put("package", "package " + javaClassReader.getDomainName() + "." + javaClassReader.getModuleName() + "." + "request;");
                        ctx.put("domain", javaClassReader.getDomainName());
                        ctx.put("module", javaClassReader.getModuleName());
                        ctx.put("target", api.getTarget());
                        ctx.put("method", api.getMethod());
                        ctx.put("importList", javaClassReader.getImportList());
                        ctx.put("annotation", javaClassReader.getAnnotationList());
                        ctx.put("className", javaClassReader.getClassName().replaceAll("Request", ""));
                        ctx.put("body", javaClassReader.getBody());
                        ctx.put("tool", tool);
                        ctx.put("hasList", javaClassReader.isHasList());
                        ctx.put("findOrSearchflag", javaClassReader.getFindOrSearchflag());
                        File file = new File(apiRequest.getAbsolutePath(), api.getRequestFile().getName());

                        freeMarkerManager.outputTemp(file, "Java_api/module/req/request.java", ctx);
                        System.out.println("生成文件" + file.getName() + "成功");
                    }
                    //endregion

                    for (String s : javaClassReader.getDeptEntList()) {
                        Pattern compile = Pattern.compile("(.*\\.ent)\\.(.*)");
                        Matcher matcher = compile.matcher(s);

                        if (matcher.find()) {
                            String Package = matcher.group(1);
                            String EntName = matcher.group(2);

                            List<File> files = Tool.find(this.module, EntName + ".java");

                            for (File file : files) {
                                EntityReader entReader = new EntityReader(file);
                                if (Package.equals(entReader.getPackageName())) {
                                    HashMap<String, Object> ctx = new HashMap<String, Object>();
                                    ctx.put("package", "package " + entReader.getDomainName() + "." + entReader.getModuleName() + "." + "request;");
                                    ctx.put("domain", entReader.getDomainName());
                                    ctx.put("module", entReader.getModuleName());
                                    ctx.put("importList", entReader.getImportList());
                                    ctx.put("annotation", entReader.getAnnotationList());
                                    ctx.put("className", entReader.getClassName().replaceAll("Entity", ""));
                                    ctx.put("body", entReader.getBody());
                                    ctx.put("tool", tool);
                                    File EntFile = new File(entity.getAbsolutePath(), EntName);
                                    freeMarkerManager.outputTemp(EntFile, "Java_api/module/ent/entity.java", ctx);
                                    System.out.println("生成文件" + EntName + "成功");

                                }
                            }

                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }

                try {
                    //region 生成response
                    JavaClassReader javaClassReader = new JavaClassReader(api.getResponseFile());
                    for (int i = 0; i < javaClassReader.getImportList().size(); i++) {
                        String import_ = javaClassReader.getImportList().get(i);
                        if (import_.contains(".module.entity.")) {
                            javaClassReader.getImportList().set(i, import_);
                        }
                    }

                    List<String> entNames = new ArrayList<>();

                    for (String s : javaClassReader.getDeptEntList()) {
                        Pattern compile = Pattern.compile("(.*\\.ent)\\.(.*)");
                        Matcher matcher = compile.matcher(s);

                        if (matcher.find()) {
                            String Package = matcher.group(1);
                            String EntName = matcher.group(2);
                            entNames.add(EntName);

                            List<File> files = Tool.find(this.module, EntName + ".java");

                            for (File file : files) {
                                EntityReader entReader = new EntityReader(file);
                                if (Package.equals(entReader.getPackageName())) {
                                    HashMap<String, Object> ctx = new HashMap<String, Object>();
                                    ctx.put("package", "package " + entReader.getDomainName() + "." + entReader.getModuleName() + "." + "request;");
                                    ctx.put("domain", entReader.getDomainName());
                                    ctx.put("module", entReader.getModuleName());
                                    ctx.put("importList", entReader.getImportList());
                                    ctx.put("annotation", entReader.getAnnotationList());
                                    ctx.put("className", entReader.getClassName().replaceAll("Entity", ""));
                                    ctx.put("body", entReader.getBody());
                                    ctx.put("tool", tool);
                                    File EntFile = new File(entity.getAbsolutePath(), EntName+".java");
                                    freeMarkerManager.outputTemp(EntFile, "Java_api/module/ent/entity.java", ctx);
                                    System.out.println("生成文件" + EntName + "成功");
                                }
                            }

                        }
                    }

                    {
                        HashMap<String, Object> ctx = new HashMap<String, Object>();
                        ctx.put("package", "package " + javaClassReader.getDomainName() + "." + javaClassReader.getModuleName() + "." + "request;");
                        ctx.put("domain", javaClassReader.getDomainName());
                        ctx.put("module", javaClassReader.getModuleName());
                        ctx.put("importList", javaClassReader.getImportList());
                        ctx.put("entNames", entNames);
                        ctx.put("annotation", javaClassReader.getAnnotationList());
                        ctx.put("className", javaClassReader.getClassName().replaceAll("Response", ""));
                        ctx.put("body", javaClassReader.getBody());
                        ctx.put("tool", tool);
                        ctx.put("hasList", javaClassReader.isHasList());
                        ctx.put("Tclass", javaClassReader.getTclass());
                        File file = new File(apiResponse.getAbsolutePath(), api.getResponseFile().getName().replaceAll("Request", "Response"));

                        freeMarkerManager.outputTemp(file, "Java_api/module/rsp/response.java", ctx);
                        System.out.println("生成文件" + api.getResponseFile().getName() + "成功");
                    }
                    //endregion


                } catch (IOException e) {
                    e.printStackTrace();
                }

//                for (String s : api.getDepReq()) {
//                    try {
//                        File f = new File(apiFile.getAbsolutePath(), s + ".java");
//                        if (!f.exists()) {
//                            System.err.println("文件" + f.getAbsolutePath() + "不存在");
//                            continue;
//                        }
//                        JavaClassReader javaClassReader = new JavaClassReader(f);
//                        for (int i = 0; i < javaClassReader.getImportList().size(); i++) {
//                            String import_ = javaClassReader.getImportList().get(i);
//                            if (import_.contains(".module.entity.")) {
//                                javaClassReader.getImportList().set(i, import_.replaceAll("\\.api\\.entity\\.", ".entity."));
//                            }
//                        }
//
//                        Method method = new Method();
//                        method.setStringMethod(api.getMethod());
//                        method.setRequest(javaClassReader.getClassName());
//                        method.setTarget(Tool.getRequestTarget(javaClassReader.getClassName()));
//                        method.setMethod(Tool.getRequestAction(javaClassReader.getClassName()));
//                        method.setManager(Tool.getRequestTarget(javaClassReader.getClassName()) + "Manager");
//
//                        {
//                            HashMap<String, Object> ctx = new HashMap<String, Object>();
//                            ctx.put("package", "package " + javaClassReader.getDomainName() + "." + javaClassReader.getModuleName() + "." + "request;");
//                            ctx.put("domain", javaClassReader.getDomainName());
//                            ctx.put("module", javaClassReader.getModuleName());
//                            ctx.put("importList", javaClassReader.getImportList());
//                            ctx.put("annotation", javaClassReader.getAnnotationList());
//                            ctx.put("className", javaClassReader.getClassName().replaceAll("Request", ""));
//                            ctx.put("body", javaClassReader.getBody());
//                            ctx.put("tool", tool);
//                            ctx.put("hasList", javaClassReader.isHasList());
//                            ctx.put("findOrSearchflag", javaClassReader.getFindOrSearchflag());
//                            File file = new File(request.getAbsolutePath(), f.getName());
//
//                            freeMarkerManager.outputTemp(file, "Java_api/module/req/request.java", ctx);
//                            System.out.println("生成文件" + file.getName() + "成功");
//                        }
//                        //endregion
//                    } catch (IOException e) {
//                        e.printStackTrace();
//                    }
//
//                    try {
//                        File f = new File(rsp.getAbsolutePath(), s.replaceAll("Request", "Response.java"));
//                        if (!f.exists()) {
//                            System.err.println("文件" + f.getAbsolutePath() + "不存在");
//                        }
//                        JavaClassReader javaClassReader = new JavaClassReader(f);
//                        for (int i = 0; i < javaClassReader.getImportList().size(); i++) {
//                            String import_ = javaClassReader.getImportList().get(i);
//                            if (import_.contains(".module.entity.")) {
//                                javaClassReader.getImportList().set(i, import_.replaceAll("\\.api\\.entity\\.", ".entity."));
//                            }
//                        }
//
//                        {
//                            HashMap<String, Object> ctx = new HashMap<String, Object>();
//                            ctx.put("package", "package " + javaClassReader.getDomainName() + "." + javaClassReader.getModuleName() + "." + "request;");
//                            ctx.put("domain", javaClassReader.getDomainName());
//                            ctx.put("module", javaClassReader.getModuleName());
//                            ctx.put("importList", javaClassReader.getImportList());
//                            ctx.put("annotation", javaClassReader.getAnnotationList());
//                            ctx.put("className", javaClassReader.getClassName().replaceAll("Response", ""));
//                            ctx.put("body", javaClassReader.getBody());
//                            ctx.put("tool", tool);
//                            ctx.put("hasList", javaClassReader.isHasList());
//                            ctx.put("Tclass", javaClassReader.getTclass());
//                            File file = new File(response.getAbsolutePath(), f.getName());
//
//                            freeMarkerManager.outputTemp(file, "Java_api/module/rsp/response.java", ctx);
//                            System.out.println("生成文件" + api.getResponseFile().getName() + "成功");
//                        }
//                        //endregion
//                    } catch (IOException e) {
//                        e.printStackTrace();
//                    }
//                }
//
//                for (String s : api.getDepEnt()) {
//
//                    try {
//                        File f = new File(ent.getAbsolutePath(), s + ".java");
//                        if (!f.exists()) {
//                            System.out.println("文件" + f.getAbsolutePath() + "不存在");
//                            continue;
//                        }
//                        JavaClassReader javaClassReader = new JavaClassReader(f);
//                        for (int i = 0; i < javaClassReader.getImportList().size(); i++) {
//                            String import_ = javaClassReader.getImportList().get(i);
//                            if (import_.contains(".module.entity.")) {
//                                javaClassReader.getImportList().set(i, import_.replaceAll("\\.api\\.entity\\.", ".entity."));
//                            }
//                        }
//
//                        {
//                            HashMap<String, Object> ctx = new HashMap<String, Object>();
//                            ctx.put("package", "package " + javaClassReader.getDomainName() + "." + javaClassReader.getModuleName() + "." + "request;");
//                            ctx.put("domain", javaClassReader.getDomainName());
//                            ctx.put("module", javaClassReader.getModuleName());
//                            ctx.put("importList", javaClassReader.getImportList());
//                            ctx.put("annotation", javaClassReader.getAnnotationList());
//                            ctx.put("className", javaClassReader.getClassName().replaceAll("Entity", ""));
//                            ctx.put("body", javaClassReader.getBody());
//                            ctx.put("tool", tool);
//                            File file = new File(entity.getAbsolutePath(), f.getName());
//                            freeMarkerManager.outputTemp(file, "Java_api/module/ent/entity.java", ctx);
//                            System.out.println("生成文件" + file.getName() + "成功");
//                        }
//                        //endregion
//                    } catch (IOException e) {
//                        e.printStackTrace();
//                    }
//                }
            }
        }


        {
            System.out.println("生成模块:Pom");
            HashMap<String, Object> ctx = new HashMap<String, Object>();
            ctx.put("module", apiFile.getName());
            File file = new File(apiFile, "pom.xml");
            freeMarkerManager.outputTemp(file, "Java_api/pom.xml", ctx);
        }

        {
            System.out.println("生成基础类");
            HashMap<String, Object> ctx = new HashMap<String, Object>();
            ctx.put("domain", javaClass.getDomainName());
            ctx.put("module", javaClass.getModuleName());
            {
                File file = new File(domain, "ApiEntity.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ApiEntity.java", ctx);
            }
            {
                File file = new File(domain, "AESUtil.java");
                freeMarkerManager.outputTemp(file, "/Java_api/AESUtil.java", ctx);
            }
            {
                File file = new File(domain, "ApiClient.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ApiClient.java", ctx);
            }
            {
                File file = new File(domain, "ApiRequest.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ApiRequest.java", ctx);
            }
            {
                File file = new File(domain, "ApiFindRequest.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ApiFindRequest.java", ctx);
            }
            {
                File file = new File(domain, "ApiSearchRequest.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ApiSearchRequest.java", ctx);
            }
            {
                File file = new File(domain, "ApiResponse.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ApiResponse.java", ctx);
            }
            {
                File file = new File(domain, "ApiFindResponse.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ApiFindResponse.java", ctx);
            }
            {
                File file = new File(domain, "Base64Util.java");
                freeMarkerManager.outputTemp(file, "/Java_api/Base64Util.java", ctx);
            }
            {
                File file = new File(domain, "DownloadUtil.java");
                freeMarkerManager.outputTemp(file, "/Java_api/DownloadUtil.java", ctx);
            }
            {
                File file = new File(domain, "Error.java");
                freeMarkerManager.outputTemp(file, "/Java_api/Error.java", ctx);
            }
            {
                File file = new File(domain, "ErrorType.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ErrorType.java", ctx);
            }
            {
                File file = new File(domain, "MapperUtil.java");
                freeMarkerManager.outputTemp(file, "/Java_api/MapperUtil.java", ctx);
            }
            {
                File file = new File(domain, "MD5Util.java");
                freeMarkerManager.outputTemp(file, "/Java_api/MD5Util.java", ctx);
            }
            {
                File file = new File(domain, "RSAUtil.java");
                freeMarkerManager.outputTemp(file, "/Java_api/RSAUtil.java", ctx);
            }
            {
                File file = new File(domain, "ProgressRequestBody.java");
                freeMarkerManager.outputTemp(file, "/Java_api/ProgressRequestBody.java", ctx);
            }

            {
                File file = new File(domain, "SortType.java");
                freeMarkerManager.outputTemp(file, "/Java_api/SortType.java", ctx);
            }

            {
                File file = new File(domain, "StringUtils.java");
                freeMarkerManager.outputTemp(file, "/Java_api/StringUtils.java", ctx);
            }

            //4.0.1 去掉验证 减少体积
//        {
//            File file = new File(frameWork , "ValidationUtil.java");
//            freeMarkerManager.outputTemp(file, "/Java_api/ValidationUtil.java", ctx);
//        }

        }

        {//生成Test
            System.out.println("生成模块:Test");
            HashMap<String, Object> ctx = new HashMap<String, Object>();
            ctx.put("tool", tool);
            ctx.put("domain", javaClass.getDomainName());
            ctx.put("module", javaClass.getModuleName());
            ctx.put("apiList", apiList);
            freeMarkerManager.outputTemp(new File(testdomain.getAbsolutePath(), "ApiTest.java"), "Java_api/ApiTest.java", ctx);
        }

        System.out.println("finish");
        return true;
    }
}
